"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const url_1 = __importDefault(require("url"));
const http_proxy_1 = __importDefault(require("http-proxy"));
const lodash_1 = __importDefault(require("lodash"));
const log = debug_1.default('koagent:http-proxy');
const isSSL = /^https|wss/i;
const isSecureProtocol = (protocol) => isSSL.test(protocol || '');
const isWS = /^ws/i;
const isWebsocketProtocol = (protocol) => isWS.test(protocol || '');
/**
 * 使用http-proxy转发请求
 */
exports.default = (options) => {
    const proxy = new http_proxy_1.default(options);
    return ({ req, res }) => new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        if (res.finished) {
            resolve();
            return;
        }
        // 如果没有设置全局的target则直接转发当前的url
        const proxyTargetUrl = req.url || '';
        log('proxyTargetUrl', proxyTargetUrl);
        const target = lodash_1.default.isString(proxyTargetUrl)
            ? url_1.default.parse(proxyTargetUrl)
            : proxyTargetUrl;
        res.setHeader('x-koagent-proxy-target', target.href || '');
        res.on('close', () => {
            reject(new Error(`Http response closed while proxying ${req.url}`));
        });
        res.on('finish', () => {
            resolve();
        });
        const proxyOptions = {
            target,
            secure: isSecureProtocol(target.protocol),
            ws: isWebsocketProtocol(target.protocol),
            changeOrigin: true,
            prependPath: false,
            followRedirects: false,
        };
        proxy.web(req, res, proxyOptions, reject);
    }));
};
//# sourceMappingURL=index.js.map